import argparse
import sys

from kmpc.version import VERSION_STR

parser = argparse.ArgumentParser()
parser.add_argument(
        "-q", "--quiet",
        help="only print errors to console log",
        action="store_true")
parser.add_argument(
        "-d", "--debug",
        help="print debug messages to console log",
        action="store_true")
parser.add_argument(
        "-n", "--newconfig",
        help="write out default config file if it doesn't exist yet",
        action="store_true")
parser.add_argument(
        "-V", "--version",
        help="print version number and exit",
        action="store_true")
parser.add_argument(
        "--helpkivy",
        help="Print Kivy's built-in argument list",
        action="store_true")


def do_args():

    # since kivy has it's own argparsing, it's necessary to do some argv
    # mangling
    args, unknown = parser.parse_known_args()
    sys.argv[1:] = unknown
    # if --helpkivy is passed, print Kivy's argument list
    if args.helpkivy:
        sys.argv.append('-h')
    # if -d/--debug is passed, use Kivy's -d flag
    if args.debug:
        sys.argv.append('-d')
    if args.version:
        print sys.argv[0]+" v"+VERSION_STR
        sys.exit(0)
    from kivy.config import Config
    if args.quiet:
        Config.set('kivy', 'log_level', 'warning')
    return args


def main_app():

    parser.add_argument(
            "--sync",
            help="run the 'sync' function with the chosen module and exit",
            choices=['all', 'music',
                     'fanart', 'exportratings',
                     'importratings'])
    args = do_args()
    from kmpc import kmpcapp
    kmpcapp.KmpcApp(args).run()


def manager_app():

    args = do_args()
    from kmpc import kmpcmanager
    kmpcmanager.ManagerApp(args).run()



import os
import io

from twisted.internet.defer import Deferred, DeferredList
import kivy
from kivy.logger import Logger

# make sure we are on updated version of kivy
kivy.require('1.10.0')


class KmpcHelpers(object):

    def formatsong(self, rec):
        """Method used by library browser to properly format a song row."""
        song = ''
        # check if there is more than one disc and display if so
        dd = rec['disc'].split('/')
        if len(dd) > 1:
            if int(dd[1]) > 1:
                song += '(Disc ' + '%02d' % int(dd[0]) + ') '
        # sometimes track numbers are like '01/05' (one of five), so drop that
        # second number
        tt = rec['track'].split('/')
        song += '%02d' % int(tt[0]) + ' '
        # if albumartist is different than track artist, display the track
        # artist
        if rec['artist'] != rec['albumartist']:
            song += rec['artist'] + ' - '
        # display the track title
        song += rec['title']
        return song

    def getfontsize(self, s, scale=1):
        """Method that determines font size based on text length."""
        # helper array for scaling font sizes based on text length
        sizearray = ['39', '38', '37', '36', '35', '34', '33', '32',
                     '31', '30', '29', '28', '27', '26', '25']
        lr = len(s)
        if lr < 33:
            rsize = '40'
        elif lr >= 55:
            rsize = '24'
        else:
            rsize = sizearray[int(round((lr-33)/21*14))]
        return format(int(round(int(rsize)/scale))) + 'sp'

    def decodeFileName(self, name):
        """Method that tries to intelligently decode a filename to handle
        unicode weirdness."""
        if type(name) == str:
            try:
                name = name.decode('utf8')
            except Exception:
                name = name.decode('windows-1252')
        return name

    def removeEmptyFolders(self, path, removeRoot=True):
        """Function to remove empty folders"""
        if not os.path.isdir(path):
            return
        # remove empty subfolders
        files = os.listdir(path)
        if len(files):
            for f in files:
                fullpath = os.path.join(path, f)
                if os.path.isdir(fullpath):
                    self.removeEmptyFolders(fullpath)
        # if folder empty, delete it
        files = os.listdir(path)
        if len(files) == 0 and removeRoot:
            Logger.debug("removeEmptyDir:" + path)
            os.rmdir(path)



from twisted.internet import protocol
import kivy
from kivy.app import App
from kivy.logger import Logger

from kmpc.mpd import MPDProtocol

# make sure we are on updated version of kivy
kivy.require('1.10.0')


class MPDFactoryProtocol(MPDProtocol):
    """Factory for MPDProtocol."""

    def connectionMade(self):
        """Call the connectionMade override."""
        if callable(self.factory.connectionMade):
            self.factory.connectionMade(self)

    def connectionLost(self, reason):
        """Call the connectionLost override."""
        if callable(self.factory.connectionLost):
            self.factory.connectionLost(self, reason)


class MPDClientFactory(protocol.ClientFactory):
    """Factory for MPDClient."""

    protocol = MPDFactoryProtocol
    connectionMade = None
    connectionLost = None

    def __init__(self, idlehandler=None, **kwargs):
        self.idlehandler = idlehandler

    def buildProtocol(self, addr):
        """Hook up protocol and idle handler."""
        Logger.debug('MPDClientFactory: buildProtocol')
        protocol = self.protocol()
        protocol.factory = self
        if callable(self.idlehandler):
            protocol.idle_result = self.idlehandler
        return protocol


class MPDReconnectingClientFactory(protocol.ReconnectingClientFactory):
    """Factory for MPDClient."""

    protocol = MPDFactoryProtocol
    connectionMade = None
    connectionLost = None

    def __init__(self, idlehandler=None, **kwargs):
        self.idlehandler = idlehandler

    def buildProtocol(self, addr):
        """Hook up protocol and idle handler."""
        Logger.debug('MPDClientFactory: buildProtocol')
        protocol = self.protocol()
        protocol.factory = self
        if callable(self.idlehandler):
            protocol.idle_result = self.idlehandler
        return protocol

#    def clientConnectionFailed(self, connector, reason):
#        """mpd connection failed for some reason."""
#        Logger.error('Connection failed - goodbye!: '+format(reason))

#    def clientConnectionLost(self, connector, reason):
#        """mpd connection lost for some reason."""
#        Logger.error('Connection lost - goodbye!: '+format(reason))


class Dummy(object):
    """Wrapper class to handle calls to mpd before the connection is set up."""

    def __getattr__(self, attr):
        Logger.debug("MpdConnection: no connection when calling "
                     + attr + " method")
        return self

    def __call__(self, *args):
        return self


class MpdConnection(object):

    def __init__(self, config, mpdhost, mpdport,
                 idlehandler=None, initconnections=[],
                 quiet=False, reconnect=True):
        self.config = config
        self.mpdhost = mpdhost
        self.mpdport = mpdport
        self.quiet = quiet
        Logger.debug("MpdConnection: connecting to " + mpdhost + ":" + mpdport)
        # set up mpd connection
        self.initconnections = initconnections
        if reconnect:
            self.factory = MPDReconnectingClientFactory(idlehandler)
        else:
            self.factory = MPDClientFactory(idlehandler)
        self.factory.connectionMade = self.mpd_connectionMade
        self.factory.connectionLost = self.mpd_connectionLost
        from twisted.internet import reactor
        self.reactor = reactor.connectTCP(mpdhost, int(mpdport), self.factory)
        self.noprotocol = Dummy()

    # this part handles calls to protocol when it hasn't been set up yet or is
    # incorrectly specified in config
    @property
    def protocol(self):
        try:
            if self.realprotocol:
                return self.realprotocol
        except AttributeError:
            Logger.debug("MpdConnection: no mpd connected")
            return self.noprotocol

    def mpd_connectionMade(self, protocol):
        """Callback when mpd is connected."""
        # copy the protocol to all the classes
        self.realprotocol = protocol
        if not self.quiet:
            Logger.info('MpdConnection: Connected to mpd server host='
                        + self.mpdhost + ' port=' + self.mpdport)
        for ic in self.initconnections:
            if callable(ic):
                ic(self)

    def mpd_connectionLost(self, protocol, reason):
        """Callback when mpd connection is lost."""
        if not self.quiet:
            Logger.warn('MpdConnection: Connection lost: %s' % reason)

    def handle_mpd_error(self, result):
        """Prints handled errors to the error log."""
        if not self.quiet:
            Logger.warn('MpdConnection: mpd error ['
                        + format(result) + ']')



import os
import pickle
from functools import partial

import musicbrainzngs
from PIL import Image as PImage
from ConfigParser import NoSectionError, NoOptionError
import kivy
from kivy.app import App
from kivy.support import install_twisted_reactor
from kivy.logger import Logger
from kivy.network.urlrequest import UrlRequest
from kivy.uix.tabbedpanel import TabbedPanel
from kivy.clock import Clock

from kmpc.version import VERSION, VERSION_STR
from kmpc.mpdfactory import MpdConnection
from kmpc.widgets import ArtistRecycleBoxLayout, ArtistRow, UneditTextInput

# make sure we are on updated version of kivy
kivy.require('1.10.0')

# sets the location of the config folder
configdir = os.path.join(os.path.expanduser('~'), ".kmpc")


class ManagerInterface(TabbedPanel):

    def __init__(self, config, **kwargs):
        TabbedPanel.__init__(self, **kwargs)
        self.config = config
        self.artist_id_hash = {}
        self.artist_name_hash = {}
        self.media_hash = {}
        self.wr_hash = {}
        self.totaldone = 0
        self.selected_row = None
        musicbrainzngs.set_useragent(
                "kmpcmanager",
                VERSION_STR,
                'https://gitlab.com/eratosthene/kmpc')
        self.fanarturl = "http://webservice.fanart.tv/v3/music/"
        self.api_key = "406b2a5af85c14b819c1c6332354b313"
        # install twisted reactor to interface with mpd
        install_twisted_reactor()
        # open mpd connection
        self.syncmpdconnection = MpdConnection(
                self.config,
                self.config.get('sync', 'synchost'),
                self.config.get('sync', 'syncmpdport'),
                None,
                [self.init_mpd])
        if not App.get_running_app().root:
            # root isn't defined yet, it must be us
            Logger.debug("ManagerInterface: running as full app")
        else:
            # root is something else
            Logger.debug("ManagerInterface: running as class")
            App.get_running_app().root.syncmpdconnection = \
                self.syncmpdconnection

    def init_mpd(self, instance):
        self.refresh_artists_from_cache()

    def refresh_artists(self):
        (self.syncmpdconnection.protocol.
            list('musicbrainz_artistid').
            addCallback(self.populate_artists).
            addErrback(self.syncmpdconnection.handle_mpd_error))

    def populate_artists(self, result):
        Logger.info("Manager: populate_artists")
        self.totaldone = 0
        waittime = 1
        self.wr_hash = {}
        for row in result:
            aids = str(row)
            for aid in aids.split('/'):
                if (aid not in self.artist_id_hash and
                        aid not in self.wr_hash and
                        len(aid) > 0):
                    Logger.debug("MusicBrainz: scheduling query in "
                                 + str(waittime)+" seconds")
                    Clock.schedule_once(partial(self.query_mb, aid), waittime)
                    waittime += 1
                    self.wr_hash[aid] = True

    def query_mb(self, aid, *args):
        Logger.info("MusicBrainz: get_artist_by_id("+aid+")")
        try:
            mbres = musicbrainzngs.get_artist_by_id(aid)
        except WebServiceError as exc:
            Logger.error("MusicBrainz: web service error "+format(exc))
        else:
            aname = mbres['artist']['name']
            Logger.debug("query_mb: result from musicbrainz for aid "
                         + aid+": "+aname)
            self.artist_id_hash[aid] = aname
            self.artist_name_hash[aname] = aid
            data = {'artist_id': aid, 'artist_name': aname}
            self.ids.artist_tab.rv.data.append(data)
            self.ids.artist_tab.rv.refresh_from_data()
            self.totaldone += 1
            self.ids.status.text = aid+' ('+str(self.totaldone)+')'

    def write_artists_to_cache(self):
        cachefile = open(os.path.join(configdir, 'artist_cache.pkl'), 'w')
        pickle.dump(
                (self.artist_id_hash, self.artist_name_hash, self.media_hash),
                cachefile,
                -1)
        cachefile.close()

    def refresh_artists_from_cache(self):
        siat = self.ids.artist_tab
        try:
            cachefile = open(os.path.join(configdir, 'artist_cache.pkl'), 'r')
            (self.artist_id_hash, self.artist_name_hash, self.media_hash) = \
                pickle.load(cachefile)
            cachefile.close()
        except IOError:
            pass
        siat.rv.data = []
        newdata = []
        for aid, aname in self.artist_id_hash.iteritems():
            try:
                has_artistbackground = \
                    self.media_hash[aid]['has_artistbackground']
            except Exception:
                has_artistbackground = False
            try:
                has_logo = self.media_hash[aid]['has_logo']
            except Exception:
                has_logo = False
            try:
                has_badge = self.media_hash[aid]['has_badge']
            except Exception:
                has_badge = False
            datum = {
                    'artist_id': aid,
                    'artist_name': aname,
                    'has_artistbackground': has_artistbackground,
                    'has_logo': has_logo,
                    'has_badge': has_badge}
            newdata.append(datum)
        self.ids.status.text = \
            'pulled '+str(len(self.artist_id_hash)) + ' lines from cache'
        siat.rv.data = sorted(newdata, key=lambda k: k['artist_name'])

    def scan_for_media(self, index):
        siat = self.ids.artist_tab
        Logger.info('Manager: scanning '
                    + siat.rv.data[index]['artist_id']
                    + 'for media')
        fa_path = self.config.get('paths', 'fanartpath')
        artistbackground_path = os.path.join(
                fa_path,
                siat.rv.data[index]['artist_id'],
                'artistbackground')
        logo_path = os.path.join(
                fa_path,
                siat.rv.data[index]['artist_id'],
                'logo')
        badge_path = os.path.join(
                fa_path,
                siat.rv.data[index]['artist_id'],
                'badge')
        siat.rv.data[index]['has_artistbackground'] = \
            os.path.isdir(artistbackground_path)
        siat.rv.data[index]['has_logo'] = os.path.isdir(logo_path)
        siat.rv.data[index]['has_badge'] = os.path.isdir(badge_path)
        self.media_hash[siat.rv.data[index]['artist_id']] = {}
        self.media_hash[
            siat.rv.data[index]['artist_id']
            ]['has_artistbackground'] = os.path.isdir(artistbackground_path)
        self.media_hash[siat.rv.data[index]['artist_id']]['has_logo'] = \
            os.path.isdir(logo_path)
        self.media_hash[siat.rv.data[index]['artist_id']]['has_badge'] = \
            os.path.isdir(badge_path)
        siat.rv.refresh_from_data()

    def scan_row_for_media(self):
        if self.selected_row is not None:
            self.scan_for_media(self.selected_row)
        self.write_artists_to_cache()

    def scan_all_for_media(self, *args):
        for idx in range(0, len(self.ids.artist_tab.rv.data)):
            self.scan_for_media(idx)
        self.write_artists_to_cache()

    def trim_image(self, filename, request, result):
        Logger.debug("trim_image: fixing "+filename)
        image = PImage.open(filename)
        # convert to RGBa before getting bounding box to account for
        # transparent pixels
        bbox = image.convert("RGBa").getbbox()
        # crop it and save
        image = image.crop(bbox)
        image.save(filename)

    def pull_art(self, index, *args):
        Logger.info('Manager: pulling art for '
                    + self.ids.artist_tab.rv.data[index]['artist_id'])
        self.ids.artist_tab.rbl.select_node(index)
        aid = self.ids.artist_tab.rv.data[index]['artist_id']
        aname = self.ids.artist_tab.rv.data[index]['artist_name']
        fa_path = self.config.get('paths', 'fanartpath')
        fanart = self.fanarturl
        api_key = self.api_key
        furl = fanart+aid+"?api_key="+api_key
        client_key = self.config.get('fanart', 'client_key')
        if client_key:
            furl = furl+"&client_key="+client_key
        Logger.debug("pull_art: querying "+furl)
        request = UrlRequest(
                url=furl,
                on_success=partial(self.pull_art2, index))

    def pull_art2(self, index, request, result):
        aid = self.ids.artist_tab.rv.data[index]['artist_id']
        aname = self.ids.artist_tab.rv.data[index]['artist_name']
        fa_path = self.config.get('paths', 'fanartpath')
        d = result
        # see if there are blacklist entries for this artist
        bl = []
        try:
            bl = self.config.get('artblacklist', aid).split(',')
        except NoSectionError:
            Logger.debug('pull_art2: no artblacklist section found')
        except NoOptionError:
            Logger.debug('pull_art2: no blacklist entries found for '+aid)
        except Exception as e:
            Logger.exception('pull_art2: '+format(e))
        else:
            Logger.debug('pull_art2: found blacklist entries for '
                         + aid+': '+format(bl))
        if 'hdmusiclogo' in d or 'artistbackground' in d or 'musiclogo' in d:
            fapath = os.path.join(fa_path, aid)
            lpath = os.path.join(fapath, "logo")
            abpath = os.path.join(fapath, "artistbackground")
            bpath = os.path.join(fapath, "badge")
            try:
                Logger.debug("pull_art2: downloading to "+fapath)
                os.mkdir(fapath)
                with open(
                        os.path.join(
                                fapath,
                                "__"+aname.replace(os.sep, '_')+"__"),
                        'w'):
                    pass
            except OSError:
                pass
            if 'hdmusiclogo' in d:
                try:
                    os.mkdir(lpath)
                except OSError:
                    pass
                for idx, img in enumerate(d['hdmusiclogo']):
                    if (not os.path.isfile(
                                os.path.join(lpath, img['id']+'.png')) and
                            not os.path.isfile(
                                    os.path.join(bpath, img['id']+'.png')) and
                            not img['id'] in bl):
                        Logger.debug("pull_art2: downloading hdmusiclogo "
                                     + img['id'])
                        fp = os.path.join(lpath, img['id']+'.png')
                        req = UrlRequest(
                                img['url'],
                                on_success=partial(self.trim_image, fp),
                                file_path=fp)
                        if self.config.getboolean('logs', 'artlog'):
                            adfile = open(
                                    os.path.join(configdir, 'artlog.txt'),
                                    'a')
                            adfile.write(
                                    os.path.join(lpath, img['id']+'.png')+"\n")
                            adfile.close()
            if 'musiclogo' in d:
                try:
                    os.mkdir(lpath)
                except OSError:
                    pass
                for idx, img in enumerate(d['musiclogo']):
                    if (not os.path.isfile(
                                os.path.join(lpath, img['id']+'.png')) and
                            not os.path.isfile(
                                    os.path.join(bpath, img['id']+'.png')) and
                            not img['id'] in bl):
                        Logger.debug("pull_art2: downloading musiclogo "
                                     + img['id'])
                        fp = os.path.join(lpath, img['id']+'.png')
                        req = UrlRequest(
                                img['url'],
                                on_success=partial(self.trim_image, fp),
                                file_path=fp)
                        if self.config.getboolean('logs', 'artlog'):
                            adfile = open(
                                    os.path.join(configdir, 'artlog.txt'),
                                    'a')
                            adfile.write(
                                    os.path.join(lpath, img['id']+'.png')+"\n")
                            adfile.close()
            if 'artistbackground' in d:
                try:
                    os.mkdir(abpath)
                except OSError:
                    pass
                for idx, img in enumerate(d['artistbackground']):
                    if (not os.path.isfile(
                                os.path.join(abpath, img['id']+'.jpg')) and
                            not img['id'] in bl):
                        Logger.debug("pull_art2: downloading artistbackground "
                                     + img['id'])
                        fp = os.path.join(abpath, img['id']+'.jpg')
                        req = UrlRequest(img['url'], file_path=fp)
                        if self.config.getboolean('logs', 'artlog'):
                            adfile = open(
                                    os.path.join(configdir, 'artlog.txt'),
                                    'a')
                            adfile.write(
                                    os.path.join(
                                            abpath,
                                            img['id']+'.jpg')+"\n")
                            adfile.close()

    def pull_art_for_row(self):
        if self.selected_row is not None:
            self.pull_art(self.selected_row)
            self.scan_for_media(self.selected_row)
        self.write_artists_to_cache()

    def pull_art_for_all(self):
        waittime = 1
        startrow = 0
        if self.selected_row is not None:
            startrow = self.selected_row
        for idx in range(startrow, len(self.ids.artist_tab.rv.data)):
            Logger.debug("FanArt.tv: scheduling query in "
                         + str(waittime)+" seconds")
            Clock.schedule_once(partial(self.pull_art, idx), waittime)
            waittime += 1
        Clock.schedule_once(self.scan_all_for_media, waittime+5)
        self.write_artists_to_cache()



import os
import sys
import io
from pkg_resources import resource_filename

import kivy
from kivy.config import Config
from kivy.app import App
from kivy.lang import Builder
from kivy.utils import get_color_from_hex

from kmpc.mpdfactory import MpdConnection
from kmpc.managerinterface import ManagerInterface

# make sure we are on updated version of kivy
kivy.require('1.10.0')

# sets the location of the config folder
configdir = os.path.join(os.path.expanduser('~'), ".kmpc")

# load the interface kv files
Builder.load_file(
        resource_filename(
                __name__,
                os.path.join('resources/kv', 'widgets.kv')))
Builder.load_file(
        resource_filename(
                __name__,
                os.path.join('resources/kv', 'library.kv')))
Builder.load_file(
        resource_filename(
                __name__,
                os.path.join('resources/kv', 'manager.kv')))


class ManagerApp(App):

    def __init__(self, args):
        self.args = args
        super(self.__class__, self).__init__()

    def build_config(self, config):
        config.setdefaults('sync', {
            'synchost': '127.0.0.1',
            'syncmpdport': '6600',
            'synclocalmusicpath': '/mnt/music',
            'synclocalfanartpath': '/mnt/fanart',
            'syncplaylist': 'synclist'
        })
        config.setdefaults('logs', {
            'artlog': False
        })
        config.setdefaults('fanart', {
            'client_key': ''
        })
        config.setdefaults('songratings', {
            'star0': 'Silence',
            'star1': 'Songs that should never be heard',
            'star2': 'Songs no one likes',
            'star3': 'Songs for certain occasions',
            'star4': 'Songs someone else likes',
            'star5': 'Filler tracks with no music',
            'star6': 'Meh track or short musical filler',
            'star7': 'Occasional listening songs',
            'star8': 'Great songs for all occasions',
            'star9': 'Best songs by an artist',
            'star10': 'Favorite songs of all time'
        })
        config.setdefaults('artblacklist', {})
        config.setdefaults('colors', {
            'button': '#00B361',
            'backdrop': '#4096FF',
            'listitem': '#4080FF',
            'listitemselected': '#FFFF00',
            'listitemcurrent': '#521C4F'
        })

    def get_color(self, c):
        cc = self.config.get('colors', c)
        try:
            # if backdrop or button, alpha channel=1, else 0.5
            if c in ['backdrop', 'button']:
                cc += 'FF'
            else:
                cc += '80'
            t = get_color_from_hex(cc)
        except Exception:
            Logger.error("Application: color "
                         + cc+" for "+c+" is invalid")
            t = (1, 1, 1, 1)
        return t

    def get_application_config(self):
        inifile = os.path.join(configdir, 'config.ini')
        return super(self.__class__, self).get_application_config(inifile)

    def build_settings(self, settings):
        settings.add_json_panel(
                'sync settings',
                self.config,
                resource_filename(
                        __name__,
                        os.path.join('resources/json',
                                     'config_manager_sync.json')))
        settings.add_json_panel(
                'log settings',
                self.config,
                resource_filename(
                        __name__,
                        os.path.join('resources/json',
                                     'config_manager_logs.json')))
        settings.add_json_panel(
                'fanart settings',
                self.config,
                resource_filename(
                        __name__,
                        os.path.join('resources/json',
                                     'config_fanart.json')))
        settings.add_json_panel(
                'song ratings',
                self.config,
                resource_filename(
                        __name__,
                        os.path.join('resources/json',
                                     'config_star.json')))
        settings.add_json_panel(
                'colors',
                self.config,
                resource_filename(
                        __name__,
                        os.path.join('resources/json',
                                     'config_colors.json')))

    def build(self):
        if not os.path.isdir(configdir):
            os.mkdir(configdir)
        # try to read existing config file
        self.config = self.load_config()
        # write out config file in case it doesn't exist yet
        self.config.write()
        if self.args.newconfig:
            sys.exit(0)
        else:
            return ManagerInterface(self.config)


if __name__ == '__main__':
    ManagerApp().run()



import os
import sys
from pkg_resources import resource_filename

import kivy
from kivy.config import Config, ConfigParser
from kivy.app import App
from kivy.logger import Logger
from kivy.lang import Builder
from kivy.utils import get_color_from_hex

from kmpc.version import VERSION_STR
from kmpc.sync import Sync
from kmpc.kmpcinterface import KmpcInterface

# make sure we are on updated version of kivy
kivy.require('1.10.0')

# sets the location of the config folder
configdir = os.path.join(os.path.expanduser('~'), ".kmpc")

# load the interface kv files
Builder.load_file(
        resource_filename(
                __name__,
                os.path.join('resources/kv', 'widgets.kv')))
Builder.load_file(
        resource_filename(
                __name__,
                os.path.join('resources/kv', 'library.kv')))
Builder.load_file(
        resource_filename(
                __name__,
                os.path.join('resources/kv', 'playlist.kv')))
Builder.load_file(
        resource_filename(
                __name__,
                os.path.join('resources/kv', 'system.kv')))
Builder.load_file(
        resource_filename(
                __name__,
                os.path.join('resources/kv', 'interface.kv')))


class KmpcApp(App):
    """The overall app class, builds the main interface widget."""

    def __init__(self, args):
        """Override kivy config values with necessary ones"""
        self.args = args
        super(self.__class__, self).__init__()

    def build_config(self, config):
        config.setdefaults('mpd', {
            'mpdhost': '127.0.0.1',
            'mpdport': '6600'
        })
        config.setdefaults('paths', {
            'musicpath': '/mnt/music',
            'fanartpath': '/mnt/fanart',
            'tmppath': '/tmp'
        })
        config.setdefaults('sync', {
            'synchost': '127.0.0.1',
            'syncmpdport': '6600',
            'syncmusicpath': '/mnt/music',
            'syncfanartpath': '/mnt/fanart',
            'synctmppath': '/tmp',
            'syncplaylist': 'synclist'
        })
        config.setdefaults('system', {
            'rpienable': '0',
            'originalyear': '1',
            'advancedtitles': '0',
            'updatecommand': 'sudo pip install -U kmpc --no-deps',
            'rebootcommand': 'sudo reboot',
            'poweroffcommand': 'sudo poweroff',
            'exportfirst': '1',
        })
        config.setdefaults('songratings', {
            'star0': 'Silence',
            'star1': 'Songs that should never be heard',
            'star2': 'Songs no one likes',
            'star3': 'Songs for certain occasions',
            'star4': 'Songs someone else likes',
            'star5': 'Filler tracks with no music',
            'star6': 'Meh track or short musical filler',
            'star7': 'Occasional listening songs',
            'star8': 'Great songs for all occasions',
            'star9': 'Best songs by an artist',
            'star10': 'Favorite songs of all time'
        })
        config.setdefaults('colors', {
            'button': '#00B361',
            'backdrop': '#4096FF',
            'listitem': '#4080FF',
            'listitemselected': '#FFFF00',
            'listitemcurrent': '#521C4F'
        })

    def get_color(self, c):
        cc = self.config.get('colors', c)
        try:
            # if backdrop or button, alpha channel=1, else 0.5
            if c in ['backdrop', 'button']:
                cc += 'FF'
            else:
                cc += '80'
            t = get_color_from_hex(cc)
        except Exception:
            Logger.error("Application: color " + cc
                         + " for " + c + " is invalid")
            t = (1, 1, 1, 1)
        return t

    def get_application_config(self):
        ini = os.path.join(configdir, 'config.ini')
        return super(self.__class__, self).get_application_config(ini)

    def build_settings(self, settings):
        settings.add_json_panel(
                'mpd settings',
                self.config,
                resource_filename(
                        __name__,
                        os.path.join('resources/json', 'config_mpd.json')))
        settings.add_json_panel(
                'path settings',
                self.config,
                resource_filename(
                        __name__,
                        os.path.join('resources/json', 'config_paths.json')))
        settings.add_json_panel(
                'sync settings',
                self.config,
                resource_filename(
                        __name__,
                        os.path.join('resources/json', 'config_sync.json')))
        settings.add_json_panel(
                'system settings',
                self.config,
                resource_filename(
                        __name__,
                        os.path.join('resources/json', 'config_system.json')))
        settings.add_json_panel(
                'song ratings',
                self.config,
                resource_filename(
                        __name__,
                        os.path.join('resources/json', 'config_star.json')))
        settings.add_json_panel(
                'colors',
                self.config,
                resource_filename(
                        __name__,
                        os.path.join('resources/json', 'config_colors.json')))

    def on_config_change(self, config, section, key, value):
        if config is self.config:
            Logger.info("Application: config entry has changed: ["
                        + section + "] " + key + "=" + value)
            if callable(self.root.settings_update):
                self.root.settings_update()

    def build(self, *args):
        """Instantiates KmpcInterface."""
        self.version_str = VERSION_STR
        if not os.path.isdir(configdir):
            os.mkdir(configdir)
        # try to read existing config file
        self.config = self.load_config()
        # write out config file in case it doesn't exist yet
        self.config.write()
        if self.args.newconfig:
            sys.exit(0)
        elif self.args.sync:
            if self.args.sync == 'all':
                if self.config.getboolean('system', 'exportfirst'):
                    s = Sync(self.config, [
                            'music', 'fanart',
                            'exportratings', 'importratings'])
                else:
                    s = Sync(self.config, [
                            'music', 'fanart',
                            'importratings', 'exportratings'])
            else:
                s = Sync(self.config, [self.args.sync])
            sys.exit(0)
        else:
            return KmpcInterface(self.config)


if __name__ == '__main__':
    # run the app!
    KmpcApp().run()



from setuptools import setup

from kmpc.version import VERSION, VERSION_STR

setup(
    version=VERSION_STR,
    download_url='https://gitlab.com/eratosthene/kmpc/archive/'
                 + VERSION_STR
                 + '.tar.gz',
    setup_requires=['setuptools>=38.4.0'],
    setup_cfg=True
)
